use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
49699333
Alders
Alders'
Andy
Anwar
ConsoleLogger
David
Easy
Everywhere
Gregory
Håkon
Hægland
Inc
Jack
LWP
MaxMind
Mohammad
Olaf
Oschwald
Params
Precious
UserAgent
Varadinsky
Wireshark
davidp
dependabot
github
goschwald
hakon
lib
mohammad
olaf
params
simbabque
sromanov
varadinsky
whitelisted
Романов
Сергей
